// SPDX-License-Identifier: MIT
pragma solidity 0.8.4;

//
import "@openzeppelin/contracts/utils/cryptography/ECDSA.sol";
import "./ERC1155URIStorage.sol";
import "@openzeppelin/contracts/proxy/utils/Initializable.sol";
import "./libraries/LibAppStorage.sol";
import "./BaseRelayRecipient.sol";

contract OnCyberScenesFacet is Initializable, BaseRelayRecipient, ERC1155URIStorage {

  using ECDSA for bytes32;
  using Counters for Counters.Counter;

  event Minted(address indexed account, uint256 indexed tokenId, uint256 indexed amount);

  function _msgSender() internal override(BaseRelayRecipient, Context) view returns (address) {

    return BaseRelayRecipient._msgSender();

  }

  function initialize(string memory _uri, address _manager, address _trustedForwarder) public initializer {

    _setURI(_uri);
    s.manager = _manager;
    s.trustedForwarder = _trustedForwarder;

  }

  function changeManager(address _newManager) public returns (bool){

    address sender = _msgSender();
    require(sender == s.manager, "NM");
    s.manager = _newManager;
    return true;

  }

  function totalSupply() public view returns (uint256) {

    return s.totalSupply.current();

  }

  function manager() public view returns (address) {

    return s.manager;

  }

  function mint(string memory _uri, uint256 _amount, bytes memory _signature) public returns (uint256 _tokenId) {

    address sender = _msgSender();

    bytes memory _message = abi.encodePacked(_uri, _amount, sender);
    address _recoveredAddress = keccak256(_message).toEthSignedMessageHash().recover(_signature);
    require(_recoveredAddress == s.manager, "NM");

    // Mint token
    _tokenId = s.totalSupply.current();
    _setTokenURI(_tokenId, _uri);
    s.totalSupply.increment();
    _mint(sender, _tokenId, _amount, "");

    emit Minted(sender, _tokenId, _amount);
    return _tokenId;

  }

}
